@@ -21,22 +21,19 @@ robot-stm:
     exclude-patterns:
         - "boot_out.txt"
         - "/data"
-    libs:    # override IOT_LIBS
-        - $IOT49/libs
-        - iot49/libs
-        - "~"
+    path: $IOT_PROJECTS
     resources:
         - pystone.py: /flash
         - copy.py
         - abc.py:
         - boot: 
-            lib: $IOT49/boards/stm32/code
+            path: my_project/code
             unpack: true
             install-dir: /flash
             include-patterns:
                 - "./**/*.py"
         - boot: 
-            lib: $IOT49/boards/stm32/code
+            lib: ~/my_own_library
             unpack: false
             install-dir: /flash
 
@@ -45,6 +42,7 @@ robot-stm:
 class DeviceConfig:
     
     def __init__(self, name, uid, spec, file):
+        """Not usually called directly. Use get_device_config(s)"""
         self._name = name
         self._uid = uid
         self._spec = spec
@@ -62,35 +60,6 @@ class DeviceConfig:
     def file(self):
         return self._file
 
-    @property
-    def resource_files(self):
-        """Returns a dict
-           path_on_mcu -> (mtime, size, path_on_host)
-        """
-        result = {}
-        sep = os.path.sep
-        for r in self.resources:
-            for f in r.files:
-                mcu_file = sep.join(f.strip(sep).split(sep)[1:]) if r.unpack else f
-                mcu_path = os.path.join(r.install_dir, mcu_file)
-                host_path = Env.expand_path(os.path.join(r.lib, f))
-                # add folders so rsync won't delete them
-                p = mcu_path
-                while p != '/':
-                    p = os.path.dirname(p)
-                    result[os.path.normpath(p)] = (0, -1, '')
-                # add the file
-                result[mcu_path] = (
-                    os.path.getmtime(host_path), 
-                    os.path.getsize(host_path), 
-                    host_path
-                )            
-        return result
-
-    @property
-    def resources(self):
-        return [ _Resource(self, r) for r in self._spec.get('resources', []) ]
-    
     def __str__(self):
         from io import StringIO
         s = StringIO()
@@ -119,51 +88,58 @@ class DeviceConfig:
         result = {}
         names = set()
         uids  = set()
-        for dir in Env.iot_device_dirs():
-            try:
-                with cd(dir):
-                    for file in glob("*.yaml") + glob("*.yml"):
-                        with open(file) as f:
-                            for name, spec in yaml.safe_load(f.read()).items():
-                                if name in names:
-                                    raise ValueError(f"File {file}: device '{name}' redefined")
-                                names.add(name)
-                                uid = spec.get('uid')
-                                if not uid:
-                                    raise ValueError(f"File {file} device '{name}': field 'uid' is mandatory")
-                                uids.add(uid)
-                                result[name] = DeviceConfig(name, uid, spec, file)
-            except FileNotFoundError as e:
-                pass
+        try:
+            with cd(Env.iot_devices()):
+                for file in glob("*.yaml") + glob("*.yml"):
+                    with open(file) as f:
+                        for name, spec in yaml.safe_load(f.read()).items():
+                            if name in names:
+                                raise ValueError(f"File {file}: device '{name}' redefined")
+                            names.add(name)
+                            uid = spec.get('uid')
+                            if not uid:
+                                raise ValueError(f"File {file} device '{name}': field 'uid' is mandatory")
+                            uids.add(uid)
+                            result[name] = DeviceConfig(name, uid, spec, file)
+        except FileNotFoundError as e:
+            # folder Env.iot_devices() does not exist
+            pass
         return result
 
-
-"""Helpers"""
-
-class _Library:
-    """Folder with resources (e.g. Python files or packages, images, etc)"""
-    
-    def __init__(self, path):
-        self._path = path
-        p = Env.expand_path(path)
-        if not os.path.isdir(p):
-            raise ValueError(f"Library: '{path}' @ '{p}' is not a directory")
-        self._resources = os.listdir(p)
-        
-    def has_resource(self, name):
-        return name in self._resources
+    @property
+    def resources(self):
+        return [ _Resource(self, r) for r in self._spec.get('resources', []) ]
     
     @property
-    def path(self):
-        return self._path
-
+    def resource_files(self):
+        """Returns a dict
+           path_on_mcu -> (mtime, size, path_on_host)
+        """
+        result = {}
+        sep = os.path.sep
+        for r in self.resources:
+            for f in r.files:
+                mcu_file = sep.join(f.strip(sep).split(sep)[1:]) if r.unpack else f
+                mcu_path = os.path.join(r.install_dir, mcu_file)
+                host_path = Env.expand_path(os.path.join(r.path, f))
+                # add folders so rsync won't delete them
+                p = mcu_path
+                while p != '/':
+                    p = os.path.dirname(p)
+                    result[os.path.normpath(p)] = (0, -1, '')
+                # add the file
+                result[mcu_path] = (
+                    os.path.getmtime(host_path), 
+                    os.path.getsize(host_path), 
+                    host_path
+                )            
+        return result
 
 class _Resource:
     """Single Resource specified in yaml file"""
 
     def __init__(self, dev, spec):
         self._dev = dev
-        self._libs_cache = {}
         if isinstance(spec, str):
             self._resource = spec
             self._param = {}
@@ -191,7 +167,7 @@ class _Resource:
         excludes = self._param.get('exclude-patterns', self._dev._spec.get('exclude-patterns', [ 'boot_out.txt' ]))
         if isinstance(includes, str): includes = [ includes ]
         if isinstance(excludes, str): excludes = [ excludes ]
-        path = os.path.join(self.lib, self.name)
+        path = os.path.join(self.path, self.name)
         if os.path.isfile(Env.expand_path(path)): return [ self.name ]
         try:
             with cd(path):
@@ -205,37 +181,31 @@ class _Resource:
             pass
         return result
 
-    def unpack(self, path):
-        """Upload directory (unpack False) or contents (unpack False)"""
-        if hasattr(self._param, 'unpack'):
+    @property
+    def unpack(self):
+        """Upload directory (unpack False) or contents (unpack True)"""
+        if 'unpack' in self._param:
             return self._param['unpack'] 
-        return os.path.isdir(path) and not os.path.isfile(os.path.join(path, '__init__.py'))
+        full_path = os.path.join(self.path, self.name)
+        return os.path.isdir(full_path) and not os.path.isfile(os.path.join(full_path, '__init__.py'))
 
     @property
     def install_dir(self):
         """Directory on mcu in which this resource is located"""
-        d = self._param.get('install-dir', self._dev._spec.get('install-dir', '/lib'))
+        d = self._param.get('install-dir', self._dev._spec.get('install-dir', '/'))
         return d if d.startswith('/') else '/' + d
 
     @property
-    def lib(self):
-        """Library (folder) where this resource is located on the host.
-        Checks libs in order & returns first match."""       
-        for lib_name in self._libs:
-            if not lib_name: continue
-            if not lib_name in self._libs_cache:
-                self._libs_cache[lib_name] = _Library(lib_name)
-            l = self._libs_cache.get(lib_name)
-            if l.has_resource(self.name):
-                return l.path
-        raise ValueError(f"Resource {self.name} not found in libraries {self._libs}")
-
-    @property
-    def _libs(self):
-        """Path of libraries to search for this resource"""
-        libs = self._param.get('lib', self._dev._spec.get('libs', Env.iot_lib_dirs()))
-        return libs if isinstance(libs, list) else [ libs ]
+    def path(self):
+        """Library (folder) where this resource is located on the host."""
+        p = self._param.get('path', self._dev._spec.get('path', Env.iot_projects()))
+        p = Env.expand_path(p)
+        if not os.path.isabs(p):
+            p = os.path.join(Env.iot_projects(), p)
+            p = Env.expand_path(p)
+        return p
 
     def __str__(self):
-        return f"{self.lib}/{self.name} -> {self.install_dir}"
-        return f"Res {self.name:22} install-dir={self.install_dir:22} lib={self.lib}"
\ No newline at end of file
+        # return f"{self.lib}/{self.name} -> {self.install_dir}"
+        return f"Res {self.name:22} install-dir={self.install_dir:22} path={self.path}"
+    
\ No newline at end of file
